import RefErl.BufferServer.Data
import RefErl.BufferServer.Data.Rope
import qualified Data.List as L
import Data.Algorithm.Diff
import Prelude hiding (length)
import qualified System.IO.UTF8 as SysIO (readFile, writeFile)
import System.Directory
import Text.Regex.Posix
import Data.List.Split


getUpdates :: [(DI, String)] -> [Update]
getUpdates ds = getUpdates0 ds 0 0

getUpdates0 :: [(DI,String)] -> Int -> Int -> [Update]
getUpdates0 [] _ _ = []
getUpdates0 ((d,s):xs) p1 p2  
-- új állapot, régi állapot a sorrend
 | d == B = getUpdates0 xs (p1 + L.length s) (p2 + L.length s)
 | d == S = Delete (Point p2) (fromString s) : getUpdates0 xs p1 p2
 | d == F = Insert (Point p1) (fromString s) : getUpdates0 xs (p1 + L.length s) (p2 + L.length s)


getUpdatesList xs ys = getUpdates (getGroupedDiff xs ys)

{-addClosedFile :: [ClosedBuffer] -> String -> String -> Integer -> ([ClosedBuffer], [Integer])
addClosedFile [] fn bfn ai = ([ClosedBuffer {baseFileName = fn, changes = URList [ClosedChange ai bfn] []}], [])
addClosedFile (x:xs) fn bfn ai 
 | baseFileName x == fn = (x' : xs, ids)
 | otherwise        	= (x : xs', ids')
  where 
 	x' = x { changes = changes' }
	changes' = addChange c (changes x)
	c = ClosedChange ai bfn
	ids = map changeId $ getRedos $ changes x 
	(xs', ids') = addClosedFile xs fn bfn ai
-}

-------------------------------------------------
-- undo-test a zárt fájlokhoz
------------------------------------------------
-- hozzáad egy változást az undo-redo listához
addChange :: Change -> URList -> URList
addChange c (URList xs ys) = URList (c:xs) []

-- | alkalmaz egy módosítást egy fingertree-n
applyUpdate :: Update -> Rope -> Rope
applyUpdate (Insert p s) r = insertChars r s p
applyUpdate (Delete p s) r = deleteChars r p (length s)

-- | invertálja a módosítást
reverseUpdate :: Update -> Update
reverseUpdate (Delete p cs) = Insert p cs
reverseUpdate (Insert p cs) = Delete p cs

undoUntilC cs [] bs _rs = return (bs, cs)
undoUntilC cs (i:is) bs@(Buffers xs ys) rs@(Refactorings ors crs) = do
	let undoUntilB' buffer = undoUntilB buffer i
	let undoUntilCB' buffer = undoUntilClosedB buffer i
	bs' <- if i `L.elem` cs
		then undoUntilC cs is bs rs
		else case L.lookup i ors of
	 		--Nothing -> case L.lookup i crs of
			Nothing -> undoUntilC cs is bs rs
			Just fs -> do 
				ys0'' <- mapM undoUntilCB' ys0
				--return (bs, [])
				case (unzip $ map undoUntilB' xs0, unzip ys0'') of 
					((xs0', is1), (ys0', is2)) -> undoUntilC (i `L.insert` cs) (is `L.union` L.nub (concat $ is1 ++ is2)) (Buffers (xs1 ++ xs0') (ys1 ++ ys0')) rs
	  		 where 
				  (ys0, ys1) = L.partition (\b -> any (\f -> f == fileName b) fs ) ys
				  (xs0, xs1) = L.partition (\b -> any (\f -> f == fileName b) fs ) xs
	return bs' 
	
{-
-- sorrend: régi -> új
getVersionUpdates :: [Change] -> IO [Change]
getVersionUpdates [] = return []
getVersionUpdates [x] = do 
	cx <- SysIO.readFile $ changeName x
	let y = head $ splitOn ".bak." $ changeName x
	cy <- SysIO.readFile y
 	let xy = AtomicChange (changeId x) $ getUpdatesList cx cy
	return [xy] -- itt kell bevenni az aktuális állapotot!
getVersionUpdates (x:y:ys) = do
	cx <- SysIO.readFile $ changeName x 
	cy <- SysIO.readFile $ changeName y  
 	let xy = AtomicChange (changeId x) $ getUpdatesList cx cy
	yys <- getVersionUpdates (y:ys)
	return $ xy : yys
-}

undoUntilClosedB :: Buffer -> Integer -> IO (Buffer, [Integer])
undoUntilClosedB b@(Buffer { undos = URList [] _ }) _ = return (b, [])
undoUntilClosedB b n = do
	str <- SysIO.readFile $ fileName b 
 	let (URList us'@(AtomicChange i u:us) rs) = undos b 
 	let (us0, us1@(hus1:tus1)) = span (\(AtomicChange i _) -> i /= n) us'
	let apply cs u = applyUpdate u cs
	let applyUs = L.foldl apply
	let mus = mergeUntil3' (updates hus1) (L.reverse us0)
	let mcs = mergeUntil3 mus us0
	let us0' = case mus of
		[] -> L.map (\(AtomicChange _ u) -> u) (us0 ++ [hus1])
		_  ->  [mus]
 	let content' = L.foldl applyUs (fromString str) us0'
	let reverseUs u = L.reverse $ map reverseUpdate u
	let reversedUs = case mus of 
		[] -> L.reverse $ map (\(AtomicChange i u) -> (AtomicChange i $ reverseUs u)) (us0 ++ [hus1])
		_  -> [hus1 { updates = reverseUs mus }]
	let us1' = case mus of 
		[] -> L.tail us1
		_  -> mcs ++ L.tail us1
	let undos' = URList us1' (reversedUs ++ rs)
	let ids = case mus of
		[]  -> map changeId us0
		_   -> []

	return (b { undos = undos', content = content' }, ids)

undoUntilB :: Buffer -> Integer -> (Buffer, [Integer])
undoUntilB b@(Buffer { undos = URList [] _ }) _ = (b, [])
undoUntilB b n = (b { undos = undos', content = content' }, ids)
 where
 	(us0, us1@(hus1:tus1)) = span (\(AtomicChange i _) -> i /= n) us'
 	content' = L.foldl applyUs (content b) us0'
	mus = mergeUntil3' (updates hus1) (L.reverse us0)
	mcs = mergeUntil3 mus us0
	us0' = case mus of
		[] -> L.map (\(AtomicChange _ u) -> u) (us0 ++ [hus1])
		_  ->  [mus]
	applyUs = L.foldl apply
	apply cs u = applyUpdate u cs
	reverseUs u = L.reverse $ map reverseUpdate u
	reversedUs = case mus of 
		[] -> L.reverse $ map (\(AtomicChange i u) -> (AtomicChange i $ reverseUs u)) (us0 ++ [hus1])
		_  -> [hus1 { updates = reverseUs mus }]
	us1' = case mus of 
		[] -> L.tail us1
		_  -> mcs ++ L.tail us1
	undos' = URList us1' (reversedUs ++ rs)
 	(URList us'@(AtomicChange i u:us) rs) = undos b 
	ids = case mus of
		[]  -> map changeId us0
		_   -> []

------------------------------------------------------------------
-- merge
------------------------------------------------------------------
------------------------------------------------------------------
---- új change-k kiszámolásához 
------------------------------------------------------------------

mergeUntil1 :: Update -> [Update] -> [Update] -> [Update]
mergeUntil1 x acc [] = L.reverse acc
mergeUntil1 x acc (y:ys) = case merge x y of
	Just u -> mergeUntil1 x' (u:acc) ys 
		where Just x' = merge' y x
		-- ^ folyamatosan egy "korban" kell lenniük, hogy értelmes legyen a vizsgálat
		-- biztosan mergelhető a másik iránbyban is, mivel fordítva az
	Nothing -> []

mergeUntil2 :: Update -> [Change] -> [Change] -> [Change]
mergeUntil2 x acc [] = L.reverse acc
mergeUntil2 x acc (y:ys) = case mergeUntil1 x [] (updates y) of
	[] -> []
	xs -> mergeUntil2 x (AtomicChange (changeId y) xs:acc) ys

mergeUntil3 :: [Update] -> [Change] -> [Change]
mergeUntil3 [] ys = ys
mergeUntil3 (x:xs) ys = case mergeUntil2 x [] ys of 
	[] -> []
	us -> mergeUntil3 xs us

merge :: Update -> Update -> Maybe Update 
merge x@(Insert p s) y@(Insert p' s') 
 | fromPoint p + length s <= fromPoint p' = Just $ Insert (Point (fromPoint p' + length s)) s'
 | fromPoint p' + length s' <= fromPoint p = Just y
 | otherwise = Nothing
merge x@(Insert p s) y@(Delete p' s') 
 | fromPoint p + length s <= fromPoint p' = Just $ Delete (Point (fromPoint p' + length s)) s'
 | fromPoint p' + length s' <= fromPoint p = Just y
 | otherwise = Nothing
merge x@(Delete p s) y@(Insert p' s') 
 | fromPoint p + length s <= fromPoint p' = Just $ Insert (Point (fromPoint p' - length s)) s'
 | fromPoint p' + length s' <= fromPoint p = Just y
 | otherwise = Nothing
merge x@(Delete p s) y@(Delete p' s') 
 | fromPoint p + length s <= fromPoint p' = Just $ Delete (Point (fromPoint p' - length s)) s'
 | fromPoint p' + length s' <= fromPoint p = Just y
 | otherwise = Nothing

------------------------------------------------------------------
---- új (refactoring) update-k kiszámolásához 
------------------------------------------------------------------

mergeUntil1' :: Update -> [Update] -> Maybe Update
mergeUntil1' x [] = Just x
mergeUntil1' x (y:ys) = case merge' y x of
	Just u  -> mergeUntil1' u ys
	Nothing -> Nothing

mergeUntil2' :: [Update] -> [Update] -> [Update] -> [Update]
mergeUntil2' [] acc _us = L.reverse acc
mergeUntil2' (x:xs) acc us = case mergeUntil1' x us of
	Just u  -> mergeUntil2' xs (u:acc) us
	Nothing -> []
	
-- change-nek fordított sorrendeben kell lennie
mergeUntil3' :: [Update] -> [Change] -> [Update]
mergeUntil3' xs [] = xs
mergeUntil3' xs (c:cs) = case mergeUntil2' xs [] (L.reverse $ updates c) of
	[] -> []
	us -> mergeUntil3' us cs

merge' :: Update -> Update -> Maybe Update 
merge' x@(Insert p s) y@(Insert p' s') 
 | fromPoint p + length s <= fromPoint p' = Just $ Insert (Point (fromPoint p' - length s)) s'
 | fromPoint p' + length s' <= fromPoint p = Just y
 | otherwise = Nothing
merge' x@(Insert p s) y@(Delete p' s') 
 | fromPoint p + length s <= fromPoint p' = Just $ Delete (Point (fromPoint p' - length s)) s'
 | fromPoint p' + length s' <= fromPoint p = Just y
 | otherwise = Nothing
merge' x@(Delete p s) y@(Insert p' s') 
 | fromPoint p + length s <= fromPoint p' = Just $ Insert (Point (fromPoint p' + length s)) s'
 | fromPoint p' + length s' <= fromPoint p = Just y
 | otherwise = Nothing
merge' x@(Delete p s) y@(Delete p' s') 
 | fromPoint p + length s <= fromPoint p' = Just $ Delete (Point (fromPoint p' + length s)) s'
 | fromPoint p' + length s' <= fromPoint p = Just y
 | otherwise = Nothing



-------------------------------------------------------------------
-------------------------------------------------------------------

