%%% -*- coding: latin-1 -*-

%%% The  contents of this  file are  subject to  the Erlang  Public License,
%%% Version  1.1, (the  "License");  you may  not  use this  file except  in
%%% compliance  with the License.  You should  have received  a copy  of the
%%% Erlang  Public License  along  with this  software.  If not,  it can  be
%%% retrieved at http://plc.inf.elte.hu/erlang/
%%%
%%% Software  distributed under  the License  is distributed  on an  "AS IS"
%%% basis, WITHOUT  WARRANTY OF ANY  KIND, either expressed or  implied. See
%%% the License  for the specific language governing  rights and limitations
%%% under the License.
%%%
%%% The Original Code is RefactorErl.
%%%
%%% The Initial Developer of the  Original Code is Etvs Lornd University.
%%% Portions created  by Etvs  Lornd University are  Copyright 2009,
%%% Etvs Lornd University. All Rights Reserved.

%%% @author Viktoria Fordos <v@fviktoria.hu>

%% -*- mode: nitrogen -*-

-module (graphs).
-compile(export_all).
-include_lib("nitrogen/include/wf.hrl").
-include("records.hrl").
-vsn("$Rev: 7190 $ ").

%%% ============================================================================
%%% HTML output

main() -> 
	nitrogen_lib:main("graphs.html").

title() -> "RefactorErl Queries".

logout()->
	nitrogen_lib:logout_link(wf:user()).

graph_control_panel()->
	LevelCell=#tablecell{body=[#label{text="Level:"},
							   #dropdown{id=graph_level,
										 options=[#option{ text="Function", 
														   value=func, 
														   selected=false},
												  #option{ text="Module", 
														   value=mod, 
														   selected=true }],
										 html_encode=true}],
						 class="graph_controll_tablecell"},
	TypeCell=#tablecell{body=[#label{text="Type:"},
							  #dropdown{id=graph_type,
										options=[#option{text="None", 
														 value=none, 
														 selected=true},
												 #option{ text="Whole graph", 
														  value=all, 
														  selected=false},
												 #option{ text="Cyclic sub- graph", 
														  value=cycles, 
														  selected=false}],
										html_encode=true}],
						class="graph_controll_tablecell"},
	SNodeModCell=#tablecell{body=[#label{text="Starting node (module)"},
								  #textbox_autocomplete {id=starting_node_module, 
														 tag=mods }],
							class="graph_controll_tablecell"},
	SNodeFunCell=#tablecell{body=[#label{text="Starting node (function)"},
								  #textbox_autocomplete {id=starting_node_function, 
														 tag=funs }],
							class="graph_controll_tablecell"},
	BtnCell=#tablecell{body=[#button{id=draw_graph,
									 text="Generate graph",
									 postback=generate_graph,
									 class=referl_button}],
					   colspan=2,
					   class=["graph_controll_tablecell", "centered"]},
	ControlTable=#table{id=graph_control_panel,
						rows=[#tablerow{cells=[LevelCell,TypeCell]},
							  #tablerow{id=gnode,
										cells=[SNodeModCell,SNodeFunCell]},
							  #tablerow{cells=[BtnCell]}]},
	%see custom_validator's definition
	wf:wire(draw_graph, graph_type, 
			#validate { validators=[#custom { text="Please, select type or starting node", 
											  tag=some_tag, 
											  function=fun custom_validator/2}]}),
	ControlTable.

functionblock_control_panel()->
	Dirs= nitrogen_helper:get_loaded_directories(),
	Items=gen_sortitem_list(Dirs),
	wf:state(fb_sources, Dirs),
	Source=fb_sources_sortblock(Items),
	Subject=#sortblock { class=["simple","fb_block"], group=subjects, 
				 connect_with_groups=directories, tag=subject, 
				 items=[]},
	wf:state(fb_subjects, []),
	HdRow=#tablerow{cells=[#tablecell{body="Functionblocks", 
									  class=["graph_controll_tablecell", "bold"]},
						   #tablecell{body="Subjects", 
									  class=["graph_controll_tablecell", "bold"]}]},
	RegexpContents=["Define by regexp: ",
					#textbox{id=user_regexp},
					#button{id=add_re, 
							text="Add", 
							postback=add_regexp, 
							class=[referl_button]}],
	RegexpRow=#tablerow{cells=[#tablecell{body=RegexpContents,
									  class="graph_controll_tablecell",
									  colspan=2,
									  align="left"}]},
	BdRow=#tablerow{cells=[#tablecell{body=Source, 
									  class="graph_controll_tablecell", 
									  valign="top"},
						   #tablecell{body=Subject, 
									  class="graph_controll_tablecell", 
									  valign="top"}]},
	CtrRow=#tablerow{cells=[#tablecell{body=#button{id=new_query_button, 
													text="Generate graph", 
													postback=generate_fb_graph, 
													class=[referl_button]}, 
									    class="graph_controll_tablecell", 
										colspan=2,
										align="center"}]},
	wf:wire(add_re, 
			user_regexp, 
			#validate { validators=[#is_required { text="Required." }]}),
	
	#table{rows=[RegexpRow, HdRow, BdRow, CtrRow], class="graph_controll_table"}.

result_panel()->
	#panel{id=result_panel, body=[]}.
%%% ============================================================================
%%% autocomplete

autocomplete_enter_event(SearchTerm, mods) ->
	Mods=case nitrogen_helper:execute_system_query("mods", 
												   {query_display_opt,
													[],
													needed_pattern,none}) of
		{result, no_result} -> [];
		{result, ResultMods} -> ResultMods
	end,
	DataM=lists:map(fun(Elem)-> 
						   {struct, [{id, list_to_binary(Elem)}, 
									 {label, list_to_binary(Elem)} , 
									 {value, list_to_binary(filename:rootname(
															  Elem)) }]} 
					end,Mods),
	List = [{struct,[{id, Id }, {label, Label}, {value, Value}]} || 
			{struct,[{id, Id }, {label, Label}, {value, Value}]} <- DataM, 
			string:str(string:to_lower(binary_to_list(Label)), 
					   string:to_lower(SearchTerm)) > 0],
	mochijson2:encode(List);

autocomplete_enter_event(SearchTerm, funs) ->
	Funs=case nitrogen_helper:execute_system_query("mods.funs", 
												   {query_display_opt,
													[],
													needed_pattern,$:}) of
		{result, no_result} -> [];
		{result, ResultFuns} -> ResultFuns
	end,	
	DataF=lists:map(fun(Elem)-> 
						   {struct, [{id, list_to_binary(Elem)}, 
									 {label, list_to_binary(Elem)} , 
									 {value, list_to_binary(Elem) }]} 
					end,Funs),
    List = [{struct,[{id, Id }, {label, Label}, {value, Value}]} || 
			{struct,[{id, Id }, {label, Label}, {value, Value}]} <- DataF, 
			string:str(string:to_lower(binary_to_list(Label)), 
					   string:to_lower(SearchTerm)) > 0],
	mochijson2:encode(List).

autocomplete_select_event({struct,[{<<"id">>,_},{<<"value">>,Value}]},_Tag)->
	wf:flash(Value),
	ok.

%%% ============================================================================
%%% validators

%Only postback the form, when at least one of those three is selected.
custom_validator(_Tag, _Value) ->
	case {wf:q(graph_type),
		  wf:q(starting_node_module), 
		  wf:q(starting_node_function)} of
		{"none", undefined, undefined} -> false;
		_ ->true
	end.

%%% ============================================================================
%%% Handlers for postback events
event(add_regexp)->
	Regexp=wf:q(user_regexp),
	List=wf:state(fb_sources)++[Regexp]--wf:state(fb_subjects),
	UniqueItems=lists:usort(List),
	wf:state(fb_sources, UniqueItems),
	Items=gen_sortitem_list(UniqueItems),
	wf:replace(src_sortblock, fb_sources_sortblock(Items));

event(generate_graph)->
	Level=wf:q(graph_level),
	Type=wf:q(graph_type),
	GnodeM=wf:q(starting_node_module),
	GnodeF=wf:q(starting_node_function),
	Gnode=case Level of
			  "func"->funs;
			  "mod"-> mods;
			  _ ->    none
		  end,

	Result=case Gnode of
		none -> nitrogen_helper:generate_dependency_graph(
							  [{level,list_to_atom(Level)}, 
							   {type, list_to_atom(Type)}],wf:user());
		mods -> nitrogen_helper:generate_dependency_graph(
							  [{level,list_to_atom(Level)},
							   {type, list_to_atom(Type)},
							   {gnode,list_to_atom(filename:basename(GnodeM,".erl"))}],
							  wf:user());
		funs -> nitrogen_helper:generate_dependency_graph(
							  [{level,list_to_atom(Level)},
							   {type, list_to_atom(Type)},
							   {gnode,GnodeF}],
							  wf:user())
	end,
	case Result of
		{error, Reason} -> create_message(Reason, error);
		_ -> create_message(Result, result)
	end;

event(generate_fb_graph)->
	case wf:state(fb_subjects) of
		FBs when is_list(FBs) andalso length(FBs)>0 -> gen_fb_graph(FBs);
		_ -> create_message("Subject is empty!", error)
	end;

event(logout) ->
	nitrogen_lib:logout(wf:user()).

%%% ============================================================================
%%% Handlers for sort events	
sort_event(subject, Items) -> 
    wf:state(fb_subjects,Items),
	ok;

sort_event(source,Items) ->
	wf:state(fb_sources, Items),
	ok.
%%% ============================================================================
%%% Hepler functions
gen_fb_graph(Subjects)->
	case nitrogen_helper:generate_fb_graph(Subjects, wf:user()) of
		{error, Reason} -> create_message(Reason, error);
		Result={_,_} -> create_message(Result, result)
	end.

create_message(Content, Type)->
	Message=case Type of
				result -> create_result_message(Content);
				_ -> create_error_message(Content)
			end,
	wf:replace(result_panel,create_message_table(Message)),
	wf:wire(result_panel,#hide{}),
	wf:wire(result_panel,#appear{}).

create_message_table(Message)->
	Cell=#tablecell{body=Message, class=["graph_controll_tablecell", "red_border"]},
	#table{id=result_panel, rows=[#tablerow{cells=[Cell]}], class="query_result_table"}.

create_error_message(Text)->
	[#p{body="Error: "++wf:to_list(Text), class="error"}].

create_result_message({DotName, SvgName})->
	[#link{text="Generated graph in .dot", 
		   title = "Download result", 
		   url="images?image="++DotName,
		   style="margin:15px; padding:5px;" },
	 #br{},
	 #br{},
	 #link{text="See generated graph in .svg", 
		   title = "See result in new window", 
		   url="javascript:newWindow('images?image="++SvgName++"');", 
		   style="margin:15px; padding:5px;" }].

fb_sources_sortblock(Items)->
	#sortblock { id=src_sortblock, class=["simple","fb_block"], group=directories, 
				 connect_with_groups=subjects, tag=source, 
				 items=Items}.

gen_sortitem_list(Items)->
	[#sortitem { tag=Item, body=Item, class="fb_block_item" } || Item <- Items, Item/=[]].
