%%% -*- coding: latin-1 -*-

%%% The  contents of this  file are  subject to  the Erlang  Public License,
%%% Version  1.1, (the  "License");  you may  not  use this  file except  in
%%% compliance  with the License.  You should  have received  a copy  of the
%%% Erlang  Public License  along  with this  software.  If not,  it can  be
%%% retrieved at http://plc.inf.elte.hu/erlang/
%%%
%%% Software  distributed under  the License  is distributed  on an  "AS IS"
%%% basis, WITHOUT  WARRANTY OF ANY  KIND, either expressed or  implied. See
%%% the License  for the specific language governing  rights and limitations
%%% under the License.
%%%
%%% The Original Code is RefactorErl.
%%%
%%% The Initial Developer of the  Original Code is Etvs Lornd University.
%%% Portions created  by Etvs  Lornd University are  Copyright 2008-2009,
%%% Etvs Lornd University. All Rights Reserved.

%%% ============================================================================
%%% Module information

%%% @doc

%%% == Implementation status ==
%%% This feature is _not_ fully implemented.

%%% @author Gabor Olah <olikas.g@gmail.com>
-module(refusr_spec).
-vsn("$Rev: 7487 $"). %for emacs"

-include_lib("stdlib/include/ms_transform.hrl").

-include("user.hrl").

-include("spec.hrl").

-export([run/1, run/3, run_tr/1]).

%% for debig purposoes only.
-export([gen_spec/3, gen_spec/1]).
-export([solve/2]).



%% =========================================================================
%% =========================================================================
%% interface

run(Mod, Name, Arity) ->
    [Func] = ?Query:exec(?Query:exec(?Mod:find(Mod)),
                         ?Mod:local(Name, Arity)),
    run(Func).

run(Func) ->
    ?SLIB:init_ets(),
    %% refusr_spec_analspec:analyzeSpecs(),
    Res = gen_spec(Func),
    %% ?SLIB:delete_ets(),
    %?d(Res),
    {refusr_spec_pp:print({Func,Res}),nothing,nothig}.

run_tr(Func) ->
    ?SLIB:init_ets(),
    gen_spec(Func).





%% =========================================================================
%% =========================================================================
%% generates specs for a fun.


gen_spec(Module, Name, Arity) ->
    [Func] = ?Query:exec(?Query:exec(?Mod:find(Module)),
                         ?Mod:local(Name, Arity)),
    gen_spec(Func).

gen_spec(Func) ->
    %?d(Func),
    Clauses = ?Query:exec(Func,
                          ?Query:seq([?Fun:definition(),
                                      ?Form:clauses()])),
    Res = case Clauses of
              [] ->
                  %% There is no body for the function
                  generate_dummy_type_for_func(Func, ?SLIB:t_any());
              [Cl] ->
                  Cs = generate_cons_cl(Cl),
                  %?d(Cs),
                  Sol = ?SOL:update(?SOL:new(), Func, 'intersect',
                                    generate_dummy_type_for_func(Func, ?SLIB:t_none())),
                  iterate_single(Cs, Func, Cl, Sol, ?DEFAULT_RECURSION_LEVEL);
                  %% case ?SOL:get_type(Sol, Cl) of
                  %%     {ok, Spec} ->
                  %%         Spec;
                  %%     {error, _X} ->
                  %%         %?d(_X),
                  %%         generate_dummy_type_for_func(Func, ?SLIB:t_none())
                  %% end;
              Cls when is_list(Cls) ->
                  Css = [ generate_cons_cl(Cl) || Cl <- Cls ],
                  NewCs = ?CS:add(?CS:new(), disj, Css),
                  %?d(NewCs),
                  Sol = ?SOL:update(?SOL:new(), Func, 'intersect',
                                    generate_dummy_type_for_func(Func, ?SLIB:t_none())),
                  iterate_mult(NewCs, Func, Cls, Sol, ?DEFAULT_RECURSION_LEVEL)
          end,
    % ?d(Func),
    % ?d(Res),
    ?SLIB:update_func_type(Func, Res).

iterate_single(Cs, Func, Cl, Sol, RecNum) ->
    OldSpec = ?SOL:get_type(Sol, Func),

    Sol2 = solve(Sol, Cs),

    NewSpec = case ?SOL:get_type(Sol2, Cl) of
                  {ok, Spec} ->
                      Spec;
                  {error, _X} ->
                      generate_dummy_type_for_func(Func, ?SLIB:t_none())
              end,
    Sol3 = ?SOL:update(Sol2, Func, 'intersect', NewSpec),
    case (OldSpec =:= NewSpec) orelse RecNum < 1 of
        true ->
            NewSpec;
        false ->
            iterate_single(Cs, Func, Cl, Sol3, RecNum - 1)
    end.



iterate_mult(Cs, Func, Cls, Sol, RecNum) ->
    OldSpec = ?SOL:get_type(Sol, Func),

    Sol2 = solve(Sol, Cs),
    %?d(Sol2),
    Types = [ case ?SOL:get_type(Sol2, Cl) of
                  {ok, Spec} -> Spec;
                  {error, _} ->  generate_dummy_type_for_func(Func, ?SLIB:t_none())
              end || Cl <- Cls ],
    Args = [ (T#type.value)#funsigvalue.args  || T <- Types ],
    GroupedArgs = ?SLIB:zipN(Args),
    %?d(GroupedArgs),
    NewArgs = [ ?SLIB:t_union(A) || A <- GroupedArgs ],
    RetTypes = [ (T#type.value)#funsigvalue.retval || T <- Types ],
    NewRetType = ?SLIB:t_union(RetTypes),

    NewSpec = ?SLIB:t_funsig(NewArgs, NewRetType),
    Sol3 = ?SOL:update(Sol2, Func, 'intersect', NewSpec),
    case (OldSpec =:= NewSpec) orelse RecNum < 1 of
        true ->
            NewSpec;
        false ->
            iterate_mult(Cs, Func, Cls, Sol3, RecNum - 1)
    end.



get_func_type(Func) ->
    ?SLIB:get_func_type(Func).

generate_dummy_type_for_func(Func, Type) ->
    FuncArity = ?Fun:arity(Func),
    FuncArgs = [ Type || _S <- lists:seq(1, FuncArity) ],
    FuncRet = Type,
    ?SLIB:t_funsig(FuncArgs, FuncRet).

generate_cons_cl(Cl) ->
    Patterns = ?Query:exec(Cl, ?Clause:patterns()),
    Guards   = ?Query:exec(Cl, ?Clause:guard()),
    Bodies   = ?Query:exec(Cl, ?Clause:body()),

    PatternRes = [ gen_cons_expr(P) || P <- Patterns ],
    GuardRes = [ gen_cons_guards(G) || G <- Guards ],
    BodyRes = [ gen_cons_expr(B) || B <- Bodies ],

    PTypes = [ P#cg_state.ret_type || P <- PatternRes ],
    %?d(BodyRes),
    RetType = (lists:last(BodyRes))#cg_state.ret_type,
    %?d(RetType),
    Fun = fun(C, Acc) -> ?CS:add(Acc, conj, C#cg_state.conset) end,
    PCons = lists:foldl(Fun, ?CS:new(), PatternRes),
    GCons = lists:foldl(Fun, ?CS:new(), GuardRes),
    BCons = lists:foldl(Fun, ?CS:new(), BodyRes),

    Cs1 = ?CS:add(?CS:new(), conj,
                 [{subt, ?TV(Cl), ?SLIB:t_funsig(PTypes, RetType)}]),
    Cs2 = ?CS:add(Cs1, conj, PCons),
    Cs3 = ?CS:add(Cs2, conj, GCons),
    _Cs4 = ?CS:add(Cs3, conj, BCons).


gen_cons_guards(Node) ->
    gcg(?Expr:type(Node), Node).


gcg([]) ->
    #cg_state{ret_type = ?SLIB:t_atom('true'),
              conset = ?CS:new()};
gcg([Node]) ->
    gcg(?Expr:type(Node), Node);
gcg(Node) ->
    gcg(?Expr:type(Node), Node).

gcg(infix_expr, Node) ->
    [Lhs, Rhs] = all_children(Node),
    case ?Expr:value(Node) of
        Disj when (Disj =:= ';') or (Disj =:= 'or') ->
            LhsRes = gcg(Lhs),
            RhsRes = gcg(Rhs),
            Cs = ?CS:add(?CS:new(), disj, [LhsRes#cg_state.conset,
                                           RhsRes#cg_state.conset]),
            #cg_state{ret_type = ?SLIB:t_bool(),
                      conset = Cs};
        Conj when (Conj =:= ',') or (Conj =:= 'and') ->
            LhsRes = gcg(Lhs),
            RhsRes = gcg(Rhs),
            Cs = ?CS:add(?CS:new(), conj, [LhsRes#cg_state.conset,
                                           RhsRes#cg_state.conset]),
            #cg_state{ret_type = ?SLIB:t_bool(),
                      conset = Cs};
        _V ->
            ResLhs = gcg(Lhs),
            ResRhs = gcg(Rhs),
            {LT, RT, RetT} = ?SLIB:type_of_infix(?Expr:value(Node)),
            Cs1 = ?CS:add(?CS:new(), conj,
                          [{subt, ResLhs#cg_state.ret_type, LT},
                           {subt, ResRhs#cg_state.ret_type, RT}]),
            Cs2 = ?CS:add(Cs1, conj, ResLhs#cg_state.conset),
            FinalCons = ?CS:add(Cs2, conj, ResRhs#cg_state.conset),
            #cg_state{ret_type = RetT,
                      conset = FinalCons}
    end;
gcg(application, Node) ->
    [NameNode | ArgList] = ?Query:exec(Node, ?Expr:children()),
    Name = ?Expr:value(NameNode),
    gcgappl(Name, ?Query:exec(ArgList, ?Expr:children()));
gcg(prefix_expr, Node) ->
    [Ch] = ?Query:exec(Node, ?Expr:children()),
    Res = gcg(Ch),
    {T, RetT} = ?SLIB:type_of_prefix(?Expr:value(Node)),
    Cs1 = ?CS:add(?CS:new(), conj,
                  [{subt, Res#cg_state.ret_type, T}]),
    FinalCons = ?CS:add(Cs1, conj, Res#cg_state.conset),
    #cg_state{ret_type = RetT,
              conset = FinalCons};
gcg(parenthesis, Node) ->
    [N] = ?Query:exec(Node, ?Expr:children()),
    gcg(N);
gcg(variable, Node) ->
    VarNode = ?Query:exec(Node,
                          ?Query:any([varref], [varbind])),
    case VarNode of
        [] ->
            io:format("Error with variable ~p in guard! Maybe the source has syntactic errors. ~n", [Node]),
            #cg_state{ret_type = ?SLIB:t_none(),
                      conset = ?CS:new()};
        [V] ->
            #cg_state{ret_type = ?TV(V),
                      conset = ?CS:new()}
    end;
gcg(_, _Node) ->
    %?d(Node),
    #cg_state{ret_type = ?SLIB:t_any(),
              conset = ?CS:new()}.


gcgappl('is_atom', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_atom()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('is_binary', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_binary()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('is_boolean', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_bool()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('is_float', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_float()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('is_function', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_function()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('is_integer', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_int()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('is_list', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_list()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('hd', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_list()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('length', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_list()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('tl', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_list()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('is_number', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_number()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('is_pid', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_pid()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('is_port', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_port()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('is_refernce', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_ref()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl('is_tuple', [Node]) ->
    Res = gcg(Node),
    Cons = ?CS:add(?CS:new(), conj,
                   [{subt, Res#cg_state.ret_type, ?SLIB:t_tuple()},
                    Res#cg_state.conset]),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = Cons};
gcgappl(_Funname, _Arglist) ->
    ?d(_Funname),
    ?d(_Arglist),
    #cg_state{ret_type = ?SLIB:t_bool(),
              conset = ?CS:new()}.



%% TODO This function is needed, because andalso and orelse declares a
%% new clause due to short-cut semantics.
all_children(Expr) ->
    Expr_value = ?Expr:value(Expr),
    if
        (Expr_value == 'andalso') or (Expr_value == 'orelse') ->
            ?Query:exec(Expr, ?Query:seq([exprcl], ?Clause:body()));
        true ->
            ?Query:exec(Expr, ?Expr:children())
    end.

%% process_exprs(Pred, [E], Cs) ->
%%     Res = Pred(E, Cs),
%%     {Res#cg_state.conset, [Res#cg_state.ret_type]};
%% process_exprs(Pred, [E | Exprs], Cs) ->
%%     Res = Pred(E, Cs),
%%     [Res#cg_state.ret_type |
%%      process_exprs(Pred, Exprs, Res#cg_state.conset)].



%%% ========================================================================
%%%  SOLVE
%%% ========================================================================

%% @doc Solves the constraint set. The algorithm is based on
%% structural decomposition of the constraint set. Thought the most
%% precise result could be reached by letting the algorithm reach its
%% fix point, there is a safety mechanism to ensure there are no
%% infinit recurion. The maximum recurion limit can be set by the
%% user, but the default is 15 iteration.
solve(Sol, CS) ->
    solve(Sol, CS, ?DEFAULT_RECURSION_LEVEL).

solve(bottom, _, _) ->
    ?SOL:bottom();
solve(Sol, _ , 0) ->
    Sol;
solve(Sol, {conj, Set}, RecNum) ->
    SolCResult = solve_conj(Sol, Set),
    case SolCResult =:= Sol of
        true ->
            Sol;
        false ->
            solve(SolCResult, {conj, Set}, RecNum - 1)  % !!! Recursion
    end;
solve(Sol, {disj, Set}, _RecNum) ->
    NewSols = [S || S <- [ solve(Sol, C) || C <- Set], S =/= bottom ],
    case NewSols of
        [] ->
            ?SOL:bottom();
        _ ->
            ?SOL:merge(NewSols)
    end;
%% ---- A <: B, where A is a type variable, B can be arbitrary
solve(Sol, {subt, ?TV(Alpha), Beta}, _RecNum) ->
    %?d(Alpha),
    %?d(Beta),
    {Sol1, SA} = map_to_concrete(Sol, ?TV(Alpha)),
    {Sol2, SB} = map_to_concrete(Sol1, Beta),
    %?d(SA), ?d(SB),
    StRet = ?SLIB:is_subtype(SA, SB),
    %?d(StRet),
    case StRet#st_state.success of
        true ->
            %?d(Sol2),
            Sol3 = ?SOL:update(Sol2, Alpha, 'intersect', StRet#st_state.type),
            %?d(Sol3),
            Sol3;
        false ->
            ?SOL:bottom()
    end;
%% ---- A is a concrete type, B can be arbitrary
solve(Sol, {subt, TAlpha, Beta}, _RecNum) ->
    %?d(TAlpha),
    %?d(Beta),
    {Sol1, SA} = map_to_concrete(Sol, TAlpha),
    {Sol2, SB} = map_to_concrete(Sol1, Beta),
    %?d(SA), ?d(SB),
    StRet = ?SLIB:is_subtype(SA, SB),
    %?d(StRet),
    case StRet#st_state.success of
        true ->
            %?d(Sol2),
            ?SOL:update(Sol2, TAlpha, intersect, SB);
        false ->
            ?SOL:bottom()
    end.

solve_conj(bottom, _) ->
    ?SOL:bottom();
solve_conj(Sol, []) ->
    Sol;
solve_conj(Sol, [C | Conj]) ->
    solve_conj( solve(Sol, C), Conj).       % !!! Recursion



map_to_concrete(Sol, ?TV(Node)) ->
    case ?SOL:get_type(Sol, Node) of
        {ok, T} ->
            map_to_concrete(Sol, T);
        {error, node_missing} ->
            case Node of
                {'$gn', func, _} ->
                    {Sol, get_func_type(Node)};
                _ ->
                    {?SOL:update(Sol, Node, 'intersect', ?SLIB:t_any()),
                     ?SLIB:t_any()}
            end
    end;
map_to_concrete(Sol, #type{kind='funsig',
                           value=#funsigvalue{args=Args,
                                              retval=RetVal}}) when is_list(Args) ->

    Fun = fun(T, AccSolIn) ->
                  {AccSolOut, NewT} = map_to_concrete(AccSolIn, T),
                  {NewT, AccSolOut}
          end,
    {FinArgs, Sol1} = lists:mapfoldl(Fun, Sol, Args),
    {Sol2, FinRetVal} = map_to_concrete(Sol1, RetVal),
    {Sol2, ?SLIB:t_funsig(FinArgs, FinRetVal)};
map_to_concrete(Sol, #type{kind='list', value=Type}) ->
    {NewSol, NewType} = map_to_concrete(Sol, Type),
    {NewSol, ?SLIB:t_list(NewType)};
map_to_concrete(Sol, #type{kind='tuple', value=Types}) when is_list(Types) ->
    Fun = fun(T, AccSolIn) ->
                  {AccSolOut, NewT} = map_to_concrete(AccSolIn, T),
                  {NewT, AccSolOut}
          end,
    {NewTypes, NewSol} = lists:mapfoldl(Fun, Sol, Types),
    {NewSol, ?SLIB:t_tuple(NewTypes)};
map_to_concrete(Sol, #type{kind='union', value=Types}) when is_list(Types)->
    Fun = fun(T, AccSolIn) ->
                  {AccSolOut, NewT} = map_to_concrete(AccSolIn, T),
                  {NewT, AccSolOut}
          end,
    {NewTypes, NewSol} = lists:mapfoldl(Fun, Sol, Types),
    {NewSol, ?SLIB:t_union(NewTypes)};
map_to_concrete(Sol, Type) ->
    {Sol, Type}.













%%% ========================================================================
%%% EXPRESSIONS

gen_cons_expr(Node) ->
    gce(?Expr:type(Node), Node).

gce(Node) ->
    gce(?Expr:type(Node), Node).

gce(integer, Node) ->
    #cg_state{ret_type = ?SLIB:t_int(?Expr:value(Node)),
              conset = ?CS:new()};
gce(float, Node) ->
    #cg_state{ret_type = ?SLIB:t_float(?Expr:value(Node)),
              conset = ?CS:new()};
gce(atom, Node) ->
    #cg_state{ret_type = ?SLIB:t_atom(?Expr:value(Node)),
              conset = ?CS:new()};
gce(string, _Node) ->
    #cg_state{ret_type = ?SLIB:t_list(?SLIB:t_int()),
              conset = ?CS:new()};
gce(joker, _Node) ->
    #cg_state{ret_type = ?SLIB:t_any(),
              conset = ?CS:new()};
gce(tuple, Node) ->
    Elements = ?Query:exec(Node, ?Expr:children()),
    Res = [ gce(E) || E <- Elements ],
    RetTypes = [ E#cg_state.ret_type || E <- Res ],
    RetCons = [ E#cg_state.conset || E <- Res ],
    FinalCons = ?CS:add(?CS:new(), conj, RetCons),
    #cg_state{ret_type = ?SLIB:t_tuple(RetTypes),
              conset = FinalCons};
gce(cons, Node) ->
    Elements = ?Query:exec(Node, ?Expr:children()),
    case Elements of
        [] ->
            #cg_state{ret_type = ?SLIB:t_nil(),
                      conset = ?CS:new()};
        _ ->
            FlatList = listElements(Elements),
            Res = [ gce(E) || E <- FlatList ],
            RetTypes = [ E#cg_state.ret_type || E <- Res ],
            RetUnion = ?SLIB:t_union(RetTypes),
            RetCons = [ E#cg_state.conset || E <- Res ],
            FinalCons = ?CS:add(?CS:new(), conj, RetCons),
            #cg_state{ret_type = ?SLIB:t_list(RetUnion),
                      conset = FinalCons}
    end;
gce(variable, Node) ->
    VarNode = ?Query:exec(Node,
                          ?Query:any([varref], [varbind])),
    case VarNode of
        [] ->
            io:format("Error with variable ~p! Maybe the source has syntactic errors. ~n", [Node]),
            #cg_state{ret_type = ?SLIB:t_none(),
                      conset = ?CS:new()};
        [V] ->
            #cg_state{ret_type = ?TV(V),
                      conset = ?CS:new()}
    end;
gce(match_expr, Node) ->
    [Lhs, Rhs] = ?Query:exec(Node, ?Expr:children()),
    ResLhs = gce(Lhs),
    ResRhs = gce(Rhs),
    Cs1 = ?CS:add(?CS:new(), conj,
                  [{subt, ResLhs#cg_state.ret_type,
                    ResRhs#cg_state.ret_type},
                   {subt, ResRhs#cg_state.ret_type,
                    ResLhs#cg_state.ret_type}]),
    Cs2 = ?CS:add(Cs1, conj, ResLhs#cg_state.conset),
    FinalCons = ?CS:add(Cs2, conj, ResRhs#cg_state.conset),
    #cg_state{ret_type = ResRhs#cg_state.ret_type,
              conset = FinalCons};
gce(application, Node) ->
    Funcs = ?Query:exec(Node, ?Expr:function()),
    case Funcs of
        [] ->
            #cg_state{ret_type = ?SLIB:t_any(),
                      conset = ?CS:new()};
        [Func] ->
            Arguments = ?Query:exec(
                           ?Query:exec(Node, ?Expr:child(2)),
                           ?Expr:children()),
            Res = [ gce(A) || A <- Arguments ],
            ArgTypes = [ E#cg_state.ret_type || E <- Res],
            ArgCons = [ E#cg_state.conset || E <- Res],

            FuncArity = ?Fun:arity(Func),
            FuncArgs = [ ?TV({Func, a, S}) || S <- lists:seq(1, FuncArity) ],
            FuncRetType = ?TV({Func, ret}),
            FunSpec = ?SLIB:t_funsig(FuncArgs, FuncRetType),

            ZippedArgs = lists:zip(ArgTypes, FuncArgs),

            Cs1 = ?CS:add(?CS:new(), conj,
                          [{subt, ?TV(Func), FunSpec},
                           {subt, FunSpec,   ?TV(Func)},
                           {subt, ?TV(Node), FuncRetType}]
                          ++ [ {subt, A, F} || {A, F} <- ZippedArgs ]),


            FinalCons = ?CS:add(Cs1, conj, ArgCons),
            #cg_state{ret_type = FuncRetType,
                      conset = FinalCons}
    end;
gce(parenthesis, Node) ->
    [N] = ?Query:exec(Node, ?Expr:children()),
    gce(N);
gce(case_expr, Node) ->
    [Head] = ?Query:exec(Node, ?Query:seq([headcl], ?Clause:body())),
    HeadRes = gce(Head),
    Clauses = ?Query:exec(Node, ?Expr:clauses()),
    PartsByClauses = [ { ?Query:exec(Cl, ?Clause:patterns()),
                         ?Query:exec(Cl, ?Clause:guard()),
                         ?Query:exec(Cl, ?Clause:body()) }
                       || Cl <- Clauses ],
    Res = [ { gce(P), gcg(G), gcb(B) }
            || {[P], G, B} <- PartsByClauses ],
    %?d(Res),
    BodyRetTypes = [ R#cg_state.ret_type || {_, _, R} <- Res ],
    %?d(BodyRetTypes),
    Fun = fun(C, Acc) -> ?CS:add(Acc, conj, C#cg_state.conset) end,
    ConsByClause = [ lists:foldl(Fun, ?CS:new(),
                             [ P, G, B ])
                     || { P, G, B } <- Res ],
    %?d(ConsByClause),
    Cons = ?CS:add(?CS:new(), disj, ConsByClause),
    %?d(Cons),
    RetTypeOfPatterns = ?SLIB:t_union([ P#cg_state.ret_type
                                        || {P, _, _} <- Res ]),
    FinalCons = ?CS:add(Cons, conj,
                        [HeadRes#cg_state.conset,
                         {subt, HeadRes#cg_state.ret_type,
                          RetTypeOfPatterns}]),
    RetType = ?SLIB:t_union(BodyRetTypes),
    %?d(RetType),
    #cg_state{ret_type = RetType,
              conset = FinalCons};
gce(infix_expr, Node) ->
    [Lhs, Rhs] = ?SLIB:all_children(Node),
    ResLhs = gce(Lhs),
    ResRhs = gce(Rhs),
    {LT, RT, RetT} = ?SLIB:type_of_infix(?Expr:value(Node)),
    Cs1 = ?CS:add(?CS:new(), conj,
                  [{subt, ResLhs#cg_state.ret_type, LT},
                   {subt, ResRhs#cg_state.ret_type, RT}]),
    Cs2 = ?CS:add(Cs1, conj, ResLhs#cg_state.conset),
    FinalCons = ?CS:add(Cs2, conj, ResRhs#cg_state.conset),
    #cg_state{ret_type = RetT,
              conset = FinalCons};
gce(prefix_expr, Node) ->
    [Ch] = ?Query:exec(Node, ?Expr:children()),
    Res = gce(Ch),
    {T, RetT} = ?SLIB:type_of_prefix(?Expr:value(Node)),
    Cs1 = ?CS:add(?CS:new(), conj,
                  [{subt, Res#cg_state.ret_type, T}]),
    FinalCons = ?CS:add(Cs1, conj, Res#cg_state.conset),
    #cg_state{ret_type = RetT,
              conset = FinalCons};

gce(_, _Node) ->
    #cg_state{ret_type = ?SLIB:t_any(),
              conset = ?CS:new()}.



gcb([BodyExpr]) ->
    gce(BodyExpr);
gcb(BodyExprs) ->
    BodyRes = [ gce(B) || B <- BodyExprs ],
    BodyCs = [ R#cg_state.conset || R <- BodyRes ],
    BodyRet = (lists:last(BodyRes))#cg_state.ret_type,
    #cg_state{ret_type = BodyRet,
              conset = BodyCs}.























listElements([Child|ChildrenList]) ->
    case ?Expr:type(Child) of
        list ->
            ?Query:exec(Child, ?Expr:children()) ++ listElements(ChildrenList);
        cons ->
            listElements(?Query:exec(Child, ?Expr:children()) ++ ChildrenList);
        _ ->
            [Child]
    end;
listElements([]) ->
    [].



